import datetime
import urllib 
import logging
from wx.lib.pubsub import Publisher

class Cash(object):
    def deposit(self, amount):
	self.free += amount
	logMessage=('Deposited $%.2f into account' % amount)
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def withdraw(self, amount):
	self.free -= amount
	logMessage=('Withdrew $%.2f from account' % amount)
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def interest(self, amount):
	self.interest_earned += amount
	self.free += amount
	logMessage=('Earned $%.2f of interest on cash' % amount)
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def get_vix(self):
	url = 'http://finance.yahoo.com/d/quotes.csv?s=^vix&f=l1'
	try:
	    self.vix = float(urllib.urlopen(url).readline())
	except:
	    return "Not Available"
    
    def transform_vix(self):
	if not self.vix:
	    self.get_vix()
	vix_average=20
	vix_factor=self.core/(100-vix_average)
	self.level = (100-self.vix)*vix_factor

class Stock(object):
    def new(self):
	self.price = 0.00
	self.price_date = ''
	self.buydate = ''
	self.selldate = ''
	self.initial_purchase_date = ''
	self.cash = 0.00
	self.cash_in = 0.00
	self.cash_out = 0.00
	self.cash_level = 0.00
	self.stock = 0.00
	self.shares = 0.00
	self.portfolio_control = 0.0
	self.buy_portfolio_control = 0.0
	self.sell_portfolio_control = 0.0
	self.rec = ""
	self.rec_amount = 0.00
	self.capital_gains = 0.0
	self.dividends = 0.0
	logMessage=('Created %s portfolio' % self.symbol)
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def buy(self, date, shares, price, commission):
	# Need to manipulate the date because wx.DateTime objects are not
	# pickleable
	pdate = datetime.date(date.GetYear(), date.GetMonth()+1, date.GetDay())
	
	stock = shares * price
	if not stock:
	    # something is not right
	    return
	if self.rec == "SELL":
	    self.rec_amount = 0.0
	excess = stock - self.rec_amount
	
	self.buydate = pdate
	self.cash -= (stock + commission)
	self.price = price
	self.shares += shares
	self.stock = self.shares * self.price
	
	self.rec = ""
	self.rec_amount = 0.0
	if not self.buy_portfolio_control:
	    # This is an initial funding (or a reboot)
	    self.portfolio_control = stock
	    self.buy_portfolio_control = stock
	    self.sell_portfolio_control = stock
	    self.initial_purchase_date = self.buydate
	    logMessage=('Bought %.5f shares of %s at $%f' % 
		    (shares, self.symbol, price))
	    Publisher().sendMessage('update.log', logMessage)
	    logging.info(logMessage)
	    return

	self.buy_portfolio_control += (0.5 * stock)
	if excess:
	    self.buy_portfolio_control += (0.5 * excess)
	
	logMessage=('Bought %.5f shares of %s at $%f' % 
		(shares, self.symbol, price))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)
	
    def sell(self, date, shares, price, commission):
	# Need to manipulate the date because wx.DateTime objects are not
	# pickleable
	pdate = datetime.date(date.GetYear(), date.GetMonth()+1, date.GetDay())
	
	stock = shares * price
	if not stock:
	    # something is not right
	    return
	if self.rec == "BUY":
	    self.rec_amount = 0.0
	excess = stock - self.rec_amount
	
	self.selldate = pdate
	self.cash += (stock - commission)
	self.price = price
	self.shares -= shares
	self.stock = self.shares * self.price
	
	self.rec = ""
	self.rec_amount = 0.0

	self.sell_portfolio_control -= (0.5 * stock)
	if excess:
	    self.sell_portfolio_control -= (0.5 * excess)

	logMessage=('Sold %.5f shares of %s at $%f' % 
		(shares, self.symbol, price))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def yahoo(self):
	url = 'http://finance.yahoo.com/d/quotes.csv?s=%s&f=l1' % self.symbol
	try:
	    self.price = float(urllib.urlopen(url).readline())
	except:
	    return "Not Available"
	self.stock = self.shares * self.price
	self.price_date = datetime.datetime.now().ctime()
	logMessage=('Updated price of %s to $%f via Yahoo! Finance' %
		(self.symbol, self.price))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def manual_price(self, price):
	self.price = price
	self.stock = self.shares * self.price
	self.price_date = datetime.datetime.now().ctime()
	logMessage=('Updated price of %s to $%f via manual update' %
		(self.symbol, self.price))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def capital_gain(self, amount):
	self.capital_gains += amount
	self.cash += amount
	self.portfolio_control -= ( 0.5 * amount)
	self.buy_portfolio_control -= ( 0.5 * amount)
	self.sell_portfolio_control -= ( 0.5 * amount)
	logMessage=('Deposited $%.f of capital gains distributions into %s portfolio' % 
		(amount, self.symbol))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)
	logMessage=('Reduced portfolio control of %s by %f due to capital gains distribution' % 
		(self.symbol, amount*0.5))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def dividend(self, amount):
	self.dividends += amount
	self.cash += amount
	self.portfolio_control -= ( 0.5 * amount)
	self.buy_portfolio_control -= ( 0.5 * amount)
	self.sell_portfolio_control -= ( 0.5 * amount)
	logMessage=('Deposited $%.2f of dividends into %s portfolio' % 
		(amount, self.symbol))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)
	logMessage=('Reduced portfolio control of %s by %f due to dividend' % 
		(self.symbol, amount*0.5))
	Publisher().sendMessage('update.log', logMessage)
	logging.info(logMessage)

    def calc(self, cash_target, delay, paramChange):
	try:
	    self.cash_level = (self.cash/(self.cash + self.stock))*100
	except:
	    self.cash_level = 0.0
	self.resistance_scale = (1 - self.cash_level/cash_target)
	if self.resistance_scale < -1:
	    self.resistance_scale = -1

	self.scaled_sell_resistance = (self.sell_resistance - 
		self.sell_resistance * self.resistance_scale)
	self.scaled_buy_resistance = (self.buy_resistance +
		self.buy_resistance * self.resistance_scale)
	if not self.buy_portfolio_control and self.cash_level:
	    self.rec = "BUY"
	    self.rec_amount = (100-cash_target)/100 * self.cash
	    return

	if self.buy_portfolio_control > self.portfolio_control:
	    self.portfolio_control = self.buy_portfolio_control
	if self.sell_portfolio_control < self.portfolio_control:
	    self.sell_portfolio_control = self.portfolio_control

	if (self.stock > self.portfolio_control and
		self.buy_portfolio_control != self.portfolio_control):
	    self.buy_portfolio_control = self.portfolio_control
	    if not paramChange:
		logMessage=('Reset buy portfolio to core portfolio control of %s' 
			% self.symbol)
		Publisher().sendMessage('update.log', logMessage)
		logging.info(logMessage)
	
	elif (self.stock < self.portfolio_control and
		self.sell_portfolio_control != self.portfolio_control):
	    self.sell_portfolio_control = self.portfolio_control
	    if not paramChange:
		logMessage=('Reset sell portfolio to core portfolio control of %s'
			% self.symbol)
		Publisher().sendMessage('update.log', logMessage)
		logging.info(logMessage)
	
	if self.stock > self.sell_portfolio_control:
	    potential_sell = (self.stock - self.sell_portfolio_control) - (
		    self.scaled_sell_resistance/100 * self.stock)
	    if (potential_sell > 0 and
		    potential_sell > self.min_sell/100 * self.stock):
		if self.cash_level < cash_target:
		    self.rec = "SELL"
		    self.rec_amount = potential_sell
		    return
		else:
		    # pull a Vealie
		    self.rec = ""
		    self.rec_amount = 0.00
		    self.sell_portfolio_control += (0.5 * potential_sell)
		    if not paramChange:
			logMessage=('Increased sell portfolio control of %s by %f' % 
				(self.symbol, 0.5 * potential_sell))
			Publisher().sendMessage('update.log', logMessage)
			logging.info(logMessage)
		    return
	    else:
		self.rec = ""
		self.rec_amount = 0.00
		return
	
	elif self.stock < self.buy_portfolio_control:
	    potential_buy = (self.buy_portfolio_control - self.stock) - (
		    self.scaled_buy_resistance/100 * self.stock)
	    if (potential_buy > 0 and
		    potential_buy > self.min_buy/100 * self.stock and
		    (datetime.date.today() - self.buydate >
			datetime.timedelta(delay))):
			if self.cash_level > cash_target:
			    self.rec = "BUY"
			    if self.cash > potential_buy:
				self.rec_amount = potential_buy
			    else:
				self.rec_amount = self.cash
			    return
			else:
			    self.rec = ""
			    self.rec_amount = 0.00
			    if int(self.cash):
				self.buy_portfolio_control -= (0.5 * potential_buy)
				if not paramChange:
				    logMessage=('Decreased buy portfolio control of %s by %f' % 
					    (self.symbol, 0.5 * potential_buy))
				    Publisher().sendMessage('update.log', logMessage)
				    logging.info(logMessage)
				return
	    else:
		self.rec = ""
		self.rec_amount = 0.00
		return

