import wx
import re
import datetime
import cStringIO
from wx.lib.pubsub import Publisher
from pkg_resources import resource_filename
import dialogs, render, tooltips

class DetailFrame(wx.Frame):
    def __init__(self, db, config, history):
	self.db=db
	self.cash=self.db['cash']
	self.portfolios=self.db['portfolios']
	self.history=history
	config.SetPath('/Frames')
	width = config.ReadInt('detail_width', 700)
	height = config.ReadInt('detail_height', 600)
	sash = config.ReadInt('detail_sash', 0)
	wx.Frame.__init__(self, None, -1,
		"Portfolio Details",
		size=(width, height))
	self.panel = wx.Panel(self, -1)
	self.split = wx.SplitterWindow(self.panel)
	self.panel1 = wx.Panel(self.split, -1)
	self.panel2 = wx.Panel(self.split, -1)
	self.split.SplitHorizontally(self.panel1, self.panel2, sash)
	self.split.SetMinimumPaneSize(20)

	iconFile = resource_filename(__name__, 'icons/bar_chart.ico')
	ib = wx.IconBundle()
	ib.AddIconFromFile(iconFile, wx.BITMAP_TYPE_ANY)
	self.SetIcons(ib)

	label = wx.StaticText(self.panel, -1,
		"Portfolio symbol")
	self.choice = wx.Choice(self.panel, -1, 
		choices=sorted(self.portfolios.keys()))
	edit_btn = wx.Button(self.panel, wx.ID_EDIT)
	edit_btn.SetToolTipString(tooltips.edit_btn)
	close_btn = wx.Button(self.panel, wx.ID_CLOSE)
	back_btn = wx.Button(self.panel, name="back", id=wx.ID_BACKWARD)
	next_btn = wx.Button(self.panel, name="next", id=wx.ID_FORWARD)

	self.details_html = dialogs.MyHtmlWindow(self.panel1, -1)
	if "gtk2" in wx.PlatformInfo:
	    self.details_html.SetStandardFonts()
	self.details_html.SetRelatedFrame(self, self.GetTitle() + ": %s")
	self.details_html.SetRelatedStatusBar(2)

	self.statusbar = self.CreateStatusBar(number=3)

	self.render = render.Render()
	
	self.log = wx.TextCtrl(self.panel2, -1,
		style=wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)

	sizer = wx.BoxSizer(wx.VERTICAL)
	vslot1 = wx.FlexGridSizer(rows=1, cols=6, hgap=5, vgap=5)
	vslot1.AddMany(
		[(label, 0, wx.ALIGN_CENTER), 
		    (self.choice, 0, wx.ALIGN_CENTER),
		    (back_btn, 0, wx.ALIGN_CENTER),
		    (next_btn, 0, wx.ALIGN_CENTER),
		    (edit_btn, 0, wx.ALIGN_CENTER),
		    (close_btn, 0, wx.ALIGN_CENTER)])
	sizer.Add(vslot1)
	sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
	sizer.Add(sizer_2, 1, wx.GROW)
	sizer_2.Add(self.split, 1, wx.GROW)
	option_sizer = wx.GridSizer(cols=1)
	sizer_2.Add(option_sizer, 0, wx.GROW)
	param_staticbox = wx.StaticBox(self.panel, -1, "Parameters")
	param_sb_sizer = wx.StaticBoxSizer(param_staticbox, wx.VERTICAL)
	choices = ['All', 'Price Updates', 'Buy', 'Sell', 'Cash In', 
		'Cash Out', 'Capital Gains', 'Dividends', 'Splits']
	self.history_rb = wx.RadioBox(self.panel, -1, "History Display",
		choices=choices, majorDimension=1, style=wx.RA_SPECIFY_COLS)
	self.history_rb.SetToolTipString(tooltips.history_rb)
	option_sizer.Add(param_sb_sizer, 0, wx.ALIGN_TOP)
	option_sizer.Add(self.history_rb, 0, wx.ALIGN_BOTTOM)
	
	p1sizer = wx.BoxSizer(wx.VERTICAL)
	p1sizer.Add(self.details_html, 1, wx.GROW)
	self.panel1.SetSizer(p1sizer)
	p2sizer = wx.BoxSizer(wx.VERTICAL)
	p2sizer.Add(self.log, 1, wx.GROW)
	self.panel2.SetSizer(p2sizer)
	
	br_label = wx.StaticText(self.panel, -1, "Buy Resistance")
	self.br_sc = wx.SpinCtrl(self.panel, -1, name="buy_resistance")
	self.br_sc.SetToolTipString(tooltips.br_sc)
	
	sr_label = wx.StaticText(self.panel, -1, "Sell Resistance")
	self.sr_sc = wx.SpinCtrl(self.panel, -1, name="sell_resistance")
	self.sr_sc.SetToolTipString(tooltips.sr_sc)

	mb_label = wx.StaticText(self.panel, -1, "Minimum Buy")
	self.mb_sc = wx.SpinCtrl(self.panel, -1, name="min_buy")
	self.mb_sc.SetToolTipString(tooltips.mb_sc)

	ms_label = wx.StaticText(self.panel, -1, "Minimum Sell")
	self.ms_sc = wx.SpinCtrl(self.panel, -1, name="min_sell")
	self.ms_sc.SetToolTipString(tooltips.ms_sc)

	param_sb_sizer.AddMany([(br_label, 0, wx.TOP, 5),
	    (self.br_sc, 0, wx.BOTTOM, 5),
	    sr_label, (self.sr_sc, 0, wx.BOTTOM, 5),
	    mb_label, (self.mb_sc, 0, wx.BOTTOM, 5),
	    ms_label, (self.ms_sc, 0, wx.BOTTOM, 5)])

	self.Bind(wx.EVT_BUTTON, self.OnClose, close_btn)

	self.Bind(wx.EVT_SPINCTRL, self.OnParameter, self.br_sc)
	self.Bind(wx.EVT_SPINCTRL, self.OnParameter, self.sr_sc)
	self.Bind(wx.EVT_SPINCTRL, self.OnParameter, self.mb_sc)
	self.Bind(wx.EVT_SPINCTRL, self.OnParameter, self.ms_sc)
	self.Bind(wx.EVT_BUTTON, self.OnEdit, edit_btn)
	self.Bind(wx.EVT_CHOICE, self.OnChoice, self.choice)
	self.Bind(wx.EVT_BUTTON, self.OnChoice, next_btn)
	self.Bind(wx.EVT_BUTTON, self.OnChoice, back_btn)
	self.Bind(wx.EVT_RADIOBOX, self.OnRadio, self.history_rb)

	Publisher().subscribe(self.OnReceiveMessage, 'update.details')
	Publisher().subscribe(self.OnClose, 'close.details')
	Publisher().subscribe(self.OnLogMessage, 'update.log')
	Publisher().subscribe(self.OnNewPort, 'update.newport')
	Publisher().subscribe(self.OnDeletePort, 'delete.port')

	if 'wxMSW' in wx.PlatformInfo:
	    acceltbl = wx.AcceleratorTable([
		(wx.ACCEL_ALT, ord('B'), back_btn.GetId()),
		(wx.ACCEL_ALT, ord('F'), next_btn.GetId()),
		(wx.ACCEL_ALT, ord('E'), edit_btn.GetId()),
		(wx.ACCEL_ALT, ord('C'), close_btn.GetId()),
		])
	    self.SetAcceleratorTable(acceltbl)
	
	self.panel.SetSizer(sizer)
	self.panel.Layout()
	self.SetMinSize(sizer.GetMinSize())
	self.details_html.SetFocus()

	self.OnChoice(event='FrameLayout')

    def UpdateHistory(self, symbol):
	self.GetRegex(symbol)
	history = cStringIO.StringIO()
	for line in open(self.history):
	    if self.regex.search(line):
		history.write(line)
	self.log.ChangeValue(history.getvalue())
	self.log.ShowPosition(self.log.GetLastPosition())

    def GetRegex(self, symbol):
	if self.history_rb.GetStringSelection() == "Price Updates":
	    self.regex=re.compile("Updated price of %s" % symbol)
	elif self.history_rb.GetStringSelection() == "Buy":
	    self.regex=re.compile("Bought .* shares of %s" % symbol)
	elif self.history_rb.GetStringSelection() == "Sell":
	    self.regex=re.compile("Sold .* shares of %s" % symbol)
	elif self.history_rb.GetStringSelection() == "Capital Gains":
	    self.regex=re.compile((
		"Deposited .* of capital gains distributions into %s" 
		    % symbol))
	elif self.history_rb.GetStringSelection() == "Dividends":
	    self.regex=re.compile((
		"Deposited .* of dividends into %s" 
		    % symbol))
	elif self.history_rb.GetStringSelection() == "Splits":
	    self.regex=re.compile("Split %s" % symbol)
	elif self.history_rb.GetStringSelection() == "Cash Out":
	    self.regex=re.compile("Siphoned .* from %s" % symbol)
	elif self.history_rb.GetStringSelection() == "Cash In":
	    self.regex=re.compile("Funded %s" % symbol)
	else:
	    self.regex=re.compile(symbol)

    def OnRadio(self, event):
	symbol = self.choice.GetStringSelection()
	self.UpdateHistory(symbol)

    def OnLogMessage(self, message):
	symbol = self.choice.GetStringSelection()
	self.GetRegex(symbol)
	if self.regex.search(message.data):
	    # Specify the string here to be the same format as in the file.
	    # This allows the string to be appended to the history display
	    # without having to reparse the file.
	    self.log.AppendText('%s %s\n' %
		    (datetime.datetime.now().strftime('%a, %d %b %Y %H:%M:%S'),
			message.data))

    def OnNewPort(self, message):
	self.db = message.data
	self.cash = self.db['cash']
	self.portfolios = self.db['portfolios']
	symbol = self.choice.GetStringSelection()
	self.choice.SetItems(sorted(self.portfolios.keys()))
	self.choice.SetStringSelection(symbol)
    
    def OnDeletePort(self, message):
	self.portfolios = message.data
	index=self.choice.GetSelection()
	symbol = self.choice.GetStringSelection()
	self.choice.SetItems(sorted(self.portfolios.keys()))
	if symbol in self.choice.GetItems():
	    self.choice.SetStringSelection(symbol)
	else:
	    self.choice.SetSelection(index)
	self.OnChoice(event='delete')
    
    def OnReceiveMessage(self, message):
	self.db = message.data
	self.cash = self.db['cash']
	self.portfolios = self.db['portfolios']
	symbol = self.choice.GetStringSelection()
	stock = self.portfolios[symbol]
	self.render.details(self.details_html, stock, self.cash)
	self.UpdateHistory(symbol)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)

    def OnParameter(self, event):
	parameterName=event.GetEventObject().GetName()
	parameterValue=float(event.GetEventObject().GetValue())
	symbol = self.choice.GetStringSelection()
	stock = self.portfolios[symbol]
	if parameterName == 'buy_resistance':
	    stock.buy_resistance = parameterValue
	elif parameterName == 'sell_resistance':
	    stock.sell_resistance = parameterValue
	elif parameterName == 'min_buy':
	    stock.min_buy= parameterValue
	elif parameterName == 'min_sell':
	    stock.min_sell= parameterValue
	# need to save portfolio control values because they may change
	# while cycling through parameter changes
	buy_pc = stock.buy_portfolio_control
	sell_pc = stock.sell_portfolio_control
	stock.calc(self.cash.level, self.db['delay'], paramChange=True)
	stock.buy_portfolio_control = buy_pc
	stock.sell_portfolio_control = sell_pc
	Publisher().sendMessage('details.sync', self.portfolios)

    def OnEdit(self, event):
	symbol = self.choice.GetStringSelection()
	stock = self.portfolios[symbol]
	edit_d = dialogs.Edit(self, -1, 'Edit', stock)
	edit_d.shares_numctrl.SetLimited(True)
	result = edit_d.ShowModal()
	if result == wx.ID_OK:
	    stock.symbol = edit_d.ticker_textctrl.GetValue()
	    stock.description = edit_d.descript_textctrl.GetValue()
	    stock.shares = edit_d.shares_numctrl.GetValue()
	    stock.portfolio_control = edit_d.cpc_numctrl.GetValue()
	    stock.buy_portfolio_control = edit_d.bpc_numctrl.GetValue()
	    stock.sell_portfolio_control = edit_d.spc_numctrl.GetValue()
	    edit_d.Destroy()
	    stock.stock = stock.shares * stock.price
	    stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    Publisher().sendMessage('details.sync', self.portfolios)
	else:
	    edit_d.Destroy()

    def OnChoice(self, event):
	if event == 'FrameLayout':
	    parameterName = 'initial'
	elif event == 'delete':
	    parameterName = 'choice'
	else:
	    parameterName=event.GetEventObject().GetName()
	if parameterName == 'next':
	    count=self.choice.GetCount()
	    index=self.choice.GetSelection()
	    if index < (count - 1):
		index += 1
	    else:
		return
	    self.choice.SetSelection(index)
	elif parameterName == 'back':
	    count=self.choice.GetCount()
	    index=self.choice.GetSelection()
	    if index > 0:
		index -= 1
	    else:
		return
	    self.choice.SetSelection(index)
	elif parameterName == 'choice':
	    index = self.choice.GetSelection()
	elif parameterName == 'initial':
	    index = 0
	    self.choice.SetSelection(index)
	symbol = self.choice.GetString(index)
	stock = self.portfolios[symbol]
	self.br_sc.SetValue(stock.buy_resistance)
	self.sr_sc.SetValue(stock.sell_resistance)
	self.mb_sc.SetValue(stock.min_buy)
	self.ms_sc.SetValue(stock.min_sell)
	self.render.details(self.details_html, stock, self.cash)
	self.statusbar.SetStatusText(
		"Buy Resistance (%%): %.2f" 
		% stock.scaled_buy_resistance, 0)
	self.statusbar.SetStatusText(
		"Sell Resistance (%%): %.2f"
		% stock.scaled_sell_resistance, 1)
	self.UpdateHistory(symbol)
   
    def OnClose(self, event):
	width, height = self.GetSize()
	sash = self.split.GetSashPosition()
	Publisher().sendMessage('details.size', (width, height, sash))
	self.Close()
