import wx
import zipfile
import shelve, dbhash
import os
import datetime
import logging
from pkg_resources import resource_filename, get_distribution
from wx.lib.wordwrap import wordwrap
from wx.lib.pubsub import Publisher
from __version__ import VERSION_STRING
import aim, details, dialogs, render, tooltips

class Frame(wx.Frame):
    def __init__(self, config, db, dbfile, logfile, help):
	sp = wx.StandardPaths.Get()
	self.userData = sp.GetUserDataDir()
	self.config = config
	self.dbfile = dbfile
	self.logfile = logfile
	self.db = db
	self.help = help
	self.config.SetPath('/Frames')
	width = self.config.ReadInt('action_width', 700)
	height = self.config.ReadInt('action_height', 600)
	sash = self.config.ReadInt('action_sash', 0)
	wx.Frame.__init__(self, None, -1,
		"Adjusted Core Cash Level Automatic Investment Management",
		size=(width, height))
	self.panel = wx.Panel(self, -1)

	self.split = wx.SplitterWindow(self.panel)
	self.panel1 = wx.Panel(self.split, -1)
	self.panel2 = wx.Panel(self.split, -1)
	self.split.SplitHorizontally(self.panel1, self.panel2, sash)
	self.split.SetMinimumPaneSize(20)

	self.recs_html = dialogs.MyHtmlWindow(self.panel1, -1)
	if "gtk2" in wx.PlatformInfo:
	    self.recs_html.SetStandardFonts()
	self.recs_html.SetRelatedFrame(self, self.GetTitle() + ": %s")
	self.recs_html.SetRelatedStatusBar(0)
	self.SetStatusBarPane(1)

	self.log = wx.TextCtrl(self.panel2, -1,
		style = wx.TE_MULTILINE|wx.TE_READONLY|wx.HSCROLL)
	wx.Log_SetActiveTarget(wx.LogTextCtrl(self.log))

	sizer = wx.BoxSizer(wx.VERTICAL)
	sizer.Add(self.split, 1, wx.GROW)
	p1sizer = wx.BoxSizer(wx.VERTICAL)
	p1sizer.Add(self.recs_html, 1, wx.GROW)
	self.panel1.SetSizer(p1sizer)
	p2sizer = wx.BoxSizer(wx.VERTICAL)
	p2sizer.Add(self.log, 1, wx.GROW)
	self.panel2.SetSizer(p2sizer)
	
	subsizer = wx.FlexGridSizer(rows=1, cols=7, hgap=5, vgap=5)
	sizer.Add(subsizer)
	
	sc_cash_label = wx.StaticText(self.panel, -1, "Cash Level (%)")
	self.sc_cash = wx.SpinCtrl(self.panel, -1, 
		min=20, max=80)
	self.sc_cash.SetToolTipString(tooltips.sc_cash)
	self.Bind(wx.EVT_SPINCTRL, self.OnCashLevel, self.sc_cash)

	sc_delay_label = wx.StaticText(self.panel, -1, "Buy delay (days)")
	self.sc_delay = wx.SpinCtrl(self.panel, -1)
	self.sc_delay.SetToolTipString(tooltips.sc_delay)
	self.Bind(wx.EVT_SPINCTRL, self.OnDelay, self.sc_delay)

	yahoo_btn = wx.Button(self.panel, wx.ID_REFRESH)
	yahoo_btn.SetToolTipString(tooltips.yahoo_btn)
	self.Bind(wx.EVT_BUTTON, self.OnUpdate, yahoo_btn)
	
	port_btn = wx.Button(self.panel, wx.ID_PROPERTIES) 
	port_btn.SetToolTipString(tooltips.port_btn)
	self.Bind(wx.EVT_BUTTON, self.OnDetails, port_btn)
	subsizer.AddMany([
	    (sc_cash_label, 0, wx.ALIGN_CENTER |wx.LEFT, 20),
	    (self.sc_cash, 0, wx.ALIGN_CENTER | wx.RIGHT, 20),
	    (sc_delay_label, 0, wx.ALIGN_CENTER),
	    (self.sc_delay, 0, wx.ALIGN_CENTER | wx.RIGHT, 20),
	    ((25, -1)),
	    (port_btn, 0, wx.ALIGN_CENTER),
	    (yahoo_btn, 0, wx.ALIGN_CENTER),
	    ])
	self.panel.SetSizer(sizer)
	self.SetMinSize(sizer.GetMinSize())
	self.recs_html.SetFocus()
	
	self.statusbar = self.CreateStatusBar(number=2)
	menubar = wx.MenuBar()
	self.render = render.Render()

	# File menu
	file_menu = wx.Menu()
	backup = file_menu.Append(-1, "&Backup...",
		"Save a backup copy of data")
	self.Bind(wx.EVT_MENU, self.OnBackup, backup)
	restore = file_menu.Append(-1, "&Restore...",
		"Restore a backup copy of data")
	self.Bind(wx.EVT_MENU, self.OnRestore, restore)
	file_menu.AppendSeparator()
	exit = file_menu.Append(wx.ID_EXIT)
	self.Bind(wx.EVT_MENU, self.OnExit, exit)
	menubar.Append(file_menu, "&File")

	# Cash menu
	cash_menu = wx.Menu()
	deposit = cash_menu.Append(-1, "&Deposit...",
		"Deposit cash into account")
	self.Bind(wx.EVT_MENU, self.OnCashFlow, deposit)

	withdraw = cash_menu.Append(-1, "&Withdraw...",
		"Withdraw cash from account")
	self.Bind(wx.EVT_MENU, self.OnCashFlow, withdraw)
	fund_port = cash_menu.Append(-1, "&Fund portfolio...",
		"Deposit funds into a portfolio")
	self.Bind(wx.EVT_MENU, self.OnPortCash, fund_port)
	siphon_port = cash_menu.Append(-1, "Si&phon Funds...",
		"Siphon funds from a portfolio")
	self.Bind(wx.EVT_MENU, self.OnPortCash, siphon_port)
	menubar.Append(cash_menu, "&Cash")

	# Portfolio menu
	port_menu = wx.Menu()
	
	new_port = port_menu.Append(wx.ID_NEW,
		help="Create a new portfolio")
	self.Bind(wx.EVT_MENU, self.OnNewport, new_port)
	buy = port_menu.Append(-1, "&Buy stock...",
		"Buy shares of portfolio stock")
	self.Bind(wx.EVT_MENU, self.OnTrade, buy)
	sell = port_menu.Append(-1, "&Sell stock...",
		"Sell shares of portfolio stock")
	self.Bind(wx.EVT_MENU, self.OnTrade, sell)
	delete = port_menu.Append(-1, "&Delete",
		"Delete portfolio")
	self.Bind(wx.EVT_MENU, self.OnDelete, delete)
	detail_menu = port_menu.Append(-1, "De&tails",
		"View details of portfolios")
	self.Bind(wx.EVT_MENU, self.OnDetails, detail_menu)
	price_update = port_menu.Append(-1, "&Manual price update",
		"Update prices via manual entry")
	self.Bind(wx.EVT_MENU, self.OnPrice, price_update)
	yahoo = port_menu.Append(-1, "&Internet price update",
		"Update prices via Yahoo!")
	self.Bind(wx.EVT_MENU, self.OnUpdate, yahoo)
	
	menubar.Append(port_menu, "P&ortfolio")

	# Distributions menu
	dist_menu = wx.Menu()
	split = dist_menu.Append(-1, "&Stock Split...",
		"Stock split")
	cap_gain = dist_menu.Append(-1, "Capital &Gains...",
		"Capital Gains distribution")
	dividend = dist_menu.Append(-1, "&Dividends...",
		"Dividend distribution")
	interest = dist_menu.Append(-1, "&Interest...",
		"Interest on cash")
	self.Bind(wx.EVT_MENU, self.OnSplit, split)
	self.Bind(wx.EVT_MENU, self.OnDistribution, cap_gain)
	self.Bind(wx.EVT_MENU, self.OnDistribution, dividend)
	self.Bind(wx.EVT_MENU, self.OnDistribution, interest)

	menubar.Append(dist_menu, "&Distributions")

	# Help
	help_menu = wx.Menu()
	help = help_menu.Append(wx.ID_HELP,
		help="Help system")
	self.Bind(wx.EVT_MENU, self.OnHelp, help)
	ex_help = help_menu.Append(-1, "Help (&External viewer)",
		"View help in Web Browser")
	self.Bind(wx.EVT_MENU, self.OnExHelp, ex_help)
	about = help_menu.Append(wx.ID_ABOUT)
	self.Bind(wx.EVT_MENU, self.OnAbout, about)
	menubar.Append(help_menu, "&Help")

	self.SetMenuBar(menubar)
	Publisher().subscribe(self.OnDetailsMessage, 'details.size')
	Publisher().subscribe(self.OnLogMessage, 'update.log')
	Publisher().subscribe(self.OnDetailsSync, 'details.sync')

	if 'wxMSW' in wx.PlatformInfo:
	    acceltbl = wx.AcceleratorTable([
		(wx.ACCEL_ALT, ord('P'), port_btn.GetId()),
		(wx.ACCEL_ALT, ord('R'), yahoo_btn.GetId()),
		])
	    self.SetAcceleratorTable(acceltbl)

	self.Display()
	
    def Display(self):
	self.cash = self.db['cash']
	self.portfolios = self.db['portfolios']
	self.sc_cash.SetValue(self.cash.core)
	self.sc_delay.SetValue(self.db['delay'])
	tickers = self.portfolios.keys()
	self.btn_ids = {}
	self.btn_ids['rec'] = {}
	self.btn_ids['info'] = {}
	for ticker in tickers:
	    self.btn_ids['rec'][ticker] = wx.NewId()
	    self.btn_ids['info'][ticker] = wx.NewId()
	    self.Bind(wx.EVT_BUTTON, self.OnRec, 
		    id=self.btn_ids['rec'][ticker])
	    self.Bind(wx.EVT_BUTTON, self.OnInfo, 
		    id=self.btn_ids['info'][ticker])
	self.statusbar.SetStatusText("Adjusted cash level (%%): %.2f" 
		% self.cash.level, 0)
	self.log.LoadFile(self.logfile)
	self.log.ShowPosition(self.log.GetLastPosition())
	self.render.recs(self.recs_html, self.db, self.btn_ids)

    def sync(self):
	self.db['portfolios'] = self.portfolios
	self.db['cash'] = self.cash
	self.db.sync()
	self.render.recs(self.recs_html, self.db, self.btn_ids)
	Publisher().sendMessage('update.details', self.db)

    def Trade(self, tradeType, stock):
	trade_d = dialogs.BuySell(tradeType, stock.symbol, stock.price)
	if tradeType == 'SELL':
	    trade_d.shares_numctrl.SetMax(stock.shares)
	    trade_d.shares_numctrl.SetLimited(True)
	result = trade_d.ShowModal()
	if result == wx.ID_OK:
	    date = trade_d.cal.GetDate()
	    shares = trade_d.shares_numctrl.GetValue()
	    price = trade_d.price_numctrl.GetValue()
	    commission = trade_d.commission_numctrl.GetValue()
	    trade_d.Destroy()
	    if tradeType == 'SELL':
		stock.sell(date, shares, price, commission)
	    elif tradeType == 'BUY':
		stock.buy(date, shares, price, commission)
	    stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    self.need_sync=True
	else:
	    trade_d.Destroy()
	    self.need_sync=False

    def OnBackup(self, event):
	cwd = os.getcwd()
	dlg = wx.FileDialog(self.panel, "Save a backup copy", 
		defaultFile='backup_%s.acclaim'  % (
		    datetime.datetime.now().strftime("%Y-%m-%d_%H%M%S")),
		wildcard="*.acclaim", style=wx.SAVE|wx.OVERWRITE_PROMPT)
	if dlg.ShowModal() == wx.ID_OK:
	    filename = dlg.GetPath()
	    if not os.path.splitext(filename)[1]:
		filename = filename + '.acclaim'
	    zf = zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED)
	    os.chdir(self.userData)
	    zf.write(os.path.relpath(self.dbfile))
	    zf.write(os.path.relpath(self.logfile))
	    zf.close()
	    os.chdir(cwd)
	dlg.Destroy()

    def OnRestore(self, event):
	dlg = wx.FileDialog(self.panel, "Restore from a backup copy",
		wildcard="*.acclaim", style=wx.OPEN)
	if dlg.ShowModal() == wx.ID_OK:
	    filename = dlg.GetPath()
	    Publisher().sendMessage('close.details')
	    self.db.close()
	    zf = zipfile.ZipFile(filename, 'r', zipfile.ZIP_DEFLATED)
	    zf.extractall(self.userData)
	    zf.close()
	    self.db=shelve.open(self.dbfile)
	    self.Display()
	dlg.Destroy()

    def OnLogMessage(self, message):
	wx.LogMessage(message.data)

    def OnDetailsMessage(self, message):
	width, height, sash = message.data
	self.config.SetPath('/Frames')
	self.config.Write('detail_width', str(width))
	self.config.Write('detail_height', str(height))
	self.config.Write('detail_sash', str(sash))

    def OnDetailsSync(self, message):
	self.portfolios = message.data
	self.sync()

    def OnHelp(self, event):
	self.help.DisplayContents()

    def OnExHelp(self, event):
	htmlFile = resource_filename(__name__, 'html/index.html')
	wx.LaunchDefaultBrowser(htmlFile)

    def OnAbout(self, event):
	info = wx.AboutDialogInfo()
	info.Name = "ACCLAIM"
	info.Version = VERSION_STRING
	info.Copyright = "Copyright (C) 2009, 2010 Glenn Johnson"
	info.Description = wordwrap(
		"ACCLAIM "
		"(Adjusted Core Cash Level Automatic Investment Management) "
		"is a program derived from Robert Lichello's AIM method. "
		"It attempts to be more responsive to market conditions "
		"and obtain a more controlled cash-to-equity ratio compared "
		"to conventional AIM."
		"\n\n"
		"ACCLAIM is written in Python and uses the wxPython GUI "
		"toolkit.",
		400, wx.ClientDC(self))
	info.Developers = ['Glenn Johnson <glennpj@gmail.com>']
	info.DocWriters = ['Glenn Johnson <glennpj@gmail.com>']
	info.Artists = [
		'VisualPharm http://www.visualpharm.com/finance_icon_set/']
	if "gtk2" in wx.PlatformInfo:
	    info.License = wordwrap("""
   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this program except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
   """,
	    800, wx.ClientDC(self))
	    info.WebSite = ("http://bitbucket.org/glennpj/acclaim/overview", 
		    "ACCLAIM Web Site")
	wx.AboutBox(info)

    def OnSplit(self, event):
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.portfolios.keys()), 
		parent=self)
	stock = self.portfolios[symbol]
	label = "%s Stock Split distribution" % symbol.upper()
	split_d = dialogs.Split('Stock Split', label)
	result = split_d.ShowModal()
	if result == wx.ID_OK:
	    newShares = split_d.split_newShares.GetValue()
	    oldShares = split_d.split_oldShares.GetValue()
	    split_d.Destroy()
	    splitFactor = float(newShares)/float(oldShares)
	    stock.shares *= splitFactor
	    if stock.yahoo() == "Not Available":
		label = "Enter price of %s\n%s" % (
			stock.symbol.upper(), stock.description)
		price_d = dialogs.Price('Price Update', label)
		result = price_d.ShowModal()
		if result == wx.ID_OK:
		    price = price_d.price_numctrl.GetValue()
		    stock.manual_price(price)
		    price_d.Destroy()
		else:
		    price_d.Destroy()
		    wx.MessageBox("Stock split transaction not entered",
			    parent=self)
		    return
	    stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    self.sync()
	    logMessage=('Split %s %d for %d' % (symbol, newShares, oldShares))
	    logging.info(logMessage)
	    wx.LogMessage(logMessage)
	    return
	else:
	    split_d.Destroy()

    def OnDistribution(self, event):
	menu_text = self.GetMenuBar().FindItemById(event.GetId()).GetLabel()
	if 'Capital Gains' in menu_text:
	    distType = 'Capital Gains'
	elif 'Dividends' in menu_text:
	    distType = 'Dividends'
	elif 'Interest' in menu_text:
	    distType = 'Interest'
	    label = "Interest on cash earned"
	if not distType == 'Interest':
	    symbol = wx.GetSingleChoice("Select a portfolio", 
		    "portfolio list", 
		    sorted(self.db['portfolios'].keys()), 
		    parent=self)
	    stock = self.portfolios[symbol]
	    label = "%s %s distribution" % (symbol.upper(), distType)
	money_d = dialogs.DepositWithdraw(distType, label)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    money_d.Destroy()
	    if 'Capital Gains' in menu_text:
		stock.capital_gain(amount)
		stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    elif 'Dividends' in menu_text:
		stock.dividend(amount)
		stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    elif 'Interest' in menu_text:
		self.cash.interest(amount)
		self.db['cash'] = self.cash
	    self.sync()
	else:
	    money_d.Destroy()

    def OnDelete(self, event):
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = self.portfolios[symbol]
	if stock.shares:
	    wx.MessageBox("You must sell all shares first", parent=self)
	    return
	self.cash.deposit(stock.cash)
	self.portfolios.pop(symbol)
	# Can not call self.sync() here because the update.details routine will
	# fail when the ticker symbol is yanked out.
	self.db['portfolios'] = self.portfolios
	self.db.sync()
	self.render.recs(self.recs_html, self.db, self.btn_ids)
	logMessage=('Deleted %s' % symbol)
	logging.info(logMessage)
	wx.LogMessage(logMessage)
	Publisher().sendMessage('delete.port', self.portfolios)

    def OnInfo(self, event):
	symbol=event.GetEventObject().GetName()
	stock = self.portfolios[symbol]
	portfolio_value = (stock.stock + stock.cash)
	cash_flow = (stock.cash_in - stock.cash_out)
	try:
	    portfolio_return = (((portfolio_value - cash_flow)/cash_flow)*100)
	except:
	    portfolio_return = 0.0
	if stock.initial_purchase_date:
	    days = (float((datetime.date.today() -
		stock.initial_purchase_date).days))
	    if days < 1:
		days = 1
	    annualized_return = (
		    (portfolio_value/cash_flow)**(365/days)-1)*100

	    if stock.stock:
		next_buy_stock = (stock.buy_portfolio_control/
			(1 + stock.scaled_buy_resistance/100 
			    + stock.min_buy/100))
		next_buy_price = (next_buy_stock/stock.shares)
		
		next_sell_stock = (-stock.sell_portfolio_control/
			(stock.scaled_sell_resistance/100
			    + stock.min_sell/100 - 1))
		next_sell_price = (next_sell_stock/stock.shares)

	win = wx.PopupTransientWindow(self)
	if stock.stock:
	    st = wx.StaticText(win, -1,
		    "The total portfolio value is $%.2f\n"
		    "The total portfolio return is %.2f%%\n"
		    "The annualized portfolio return is %.2f%%\n"
		    "The next approximate buy price is $%.2f\n"
		    "The current price is $%.2f\n"
		    "The next approximate sell price is $%.2f"
		    % ( portfolio_value,
			portfolio_return,
			annualized_return,
			next_buy_price,
			stock.price,
			next_sell_price),
		    pos=(10,10))
	else:
	    st = wx.StaticText(win, -1,
		    "There is no information available.",
		    pos=(10,10))
	size = st.GetBestSize()
	win.SetSize((size.width+20, size.height+20))
	btn = event.GetEventObject()
	pos = btn.ClientToScreen( (0,0) )
	sz = btn.GetSize()
	win.Position(pos, (0, sz[1]))
	win.Popup()

    def OnRec(self, event):
	symbol=event.GetEventObject().GetName()
	tradeType=event.GetEventObject().GetLabel()
	stock=self.portfolios[symbol]
	self.Trade(tradeType, stock)
	if self.need_sync:
	    self.sync()

    def OnTrade(self, event):
	menu_text = self.GetMenuBar().FindItemById(event.GetId()).GetLabel()
	if 'Sell' in menu_text:
	    tradeType = 'SELL'
	elif 'Buy' in menu_text:
	    tradeType = 'BUY'
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = self.portfolios[symbol]
	if not stock.rec == tradeType:
	    dlg = wx.MessageDialog(self, "There is no %s signal. Continue?"
		    % tradeType,
		    'Are you sure?', 
		    wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)
	    retCode = dlg.ShowModal()
	    if not retCode == wx.ID_YES:
		dlg.Destroy()
		return
	    else:
		dlg.Destroy()
	self.Trade(tradeType, stock)
	if self.need_sync:
	    self.sync()

    def OnPortCash(self, event):
	menu_text = self.GetMenuBar().FindItemById(event.GetId()).GetLabel()
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		sorted(self.db['portfolios'].keys()), 
		parent=self)
	stock = self.portfolios[symbol]
	if 'Siphon' in menu_text:
	    cashType = 'Siphon'
	    label = "Enter amount to take from %s\n($%.2f available)" % (
		    stock.symbol.upper(), stock.cash)
	elif 'Fund' in menu_text:
	    cashType = 'Fund'
	    label = "Enter amount to fund %s\n($%.2f available)" % (
		    stock.symbol.upper(), self.cash.free)
	money_d = dialogs.DepositWithdraw(cashType, label)
	if cashType == 'Siphon':
	    money_d.money_numctrl.SetMax(stock.cash)
	elif cashType == 'Fund':
	    money_d.money_numctrl.SetMax(self.cash.free)
	money_d.money_numctrl.SetLimited(True)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    money_d.Destroy()
	    if cashType == 'Siphon':
		stock.cash_out += amount
		stock.cash -= amount
		self.cash.deposit(amount)
	    elif cashType == 'Fund':
		stock.cash_in += amount
		stock.cash += amount
		self.cash.withdraw(amount)
	    stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    self.sync()
	    if cashType == 'Siphon':
		logMessage=('Siphoned $%.2f from %s' % (amount, symbol))
	    elif cashType == 'Fund':
		logMessage=('Funded %s with $%.2f' % (symbol, amount))
	    logging.info(logMessage)
	    wx.LogMessage(logMessage)
	else:
	    money_d.Destroy()

    def OnNewport(self, event):
	stock = aim.Stock()
	mlabel = "Initial funding\n($%.2f available)" % self.cash.free
	newport_d = dialogs.NewPort(mlabel)
	newport_d.money_numctrl.SetMax(self.cash.free)
	newport_d.money_numctrl.SetLimited(True)
	result = newport_d.ShowModal()
	if result == wx.ID_OK:
	    stock.symbol = newport_d.ticker_textctrl.GetValue()
	    stock.description = newport_d.descript_textctrl.GetValue()
	    amount = newport_d.money_numctrl.GetValue()
	    br = newport_d.br_sc.GetValue()
	    sr = newport_d.sr_sc.GetValue()
	    mb = newport_d.mb_sc.GetValue()
	    ms = newport_d.ms_sc.GetValue()
	    newport_d.Destroy()
	    self.btn_ids['rec'][stock.symbol] = wx.NewId()
	    self.Bind(wx.EVT_BUTTON, self.OnRec, 
		    id=self.btn_ids['rec'][stock.symbol])
	    self.btn_ids['info'][stock.symbol] = wx.NewId()
	    self.Bind(wx.EVT_BUTTON, self.OnInfo, 
		    id=self.btn_ids['info'][stock.symbol])
	    self.cash.withdraw(amount)
	    stock.new()
	    stock.buy_resistance = br
	    stock.sell_resistance = sr
	    stock.min_buy = mb
	    stock.min_sell = ms
	    stock.cash_in += amount
	    stock.cash += amount
	    stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    self.portfolios[stock.symbol] = stock
	    self.sync()
	    logMessage=('Funded %s with $%.2f' % (stock.symbol, amount))
	    logging.info(logMessage)
	    wx.LogMessage(logMessage)
	    Publisher().sendMessage('update.newport', self.db)
	else:
	    newport_d.Destroy()
	    return

    def OnUpdate(self, event):
	if self.cash.get_vix() == "Not Available":
	    label = "Enter value of VIX\nCBOE VOLATILITY INDEX"
	    price_d = dialogs.Price('VIX Update', label)
	    result = price_d.ShowModal()
	    if result == wx.ID_OK:
		self.cash.vix = price_d.price_numctrl.GetValue()
		price_d.Destroy()
	    else:
		price_d.Destroy()
		return
	self.cash.transform_vix()
	self.statusbar.SetStatusText("Adjusted cash level (%%): %.2f" 
		% self.cash.level, 0)
	self.db['cash']=self.cash
	self.db.sync()
	list = sorted(self.portfolios.keys())
	tickers = len(list)
	dialog = wx.ProgressDialog("Updating prices",
		"",  tickers, parent=self)
	keepGoing = True
	count = 0
	while keepGoing and count < tickers:
	    ticker = list[count]
	    stock = self.portfolios[ticker]
	    if stock.rec:
		dlg = wx.MessageDialog(self, tooltips.update_warning 
			% (stock.rec, stock.symbol.upper()),
			"Are you sure?",
			wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
		retCode = dlg.ShowModal()
		if not retCode == wx.ID_YES:
		    dlg.Destroy()
		    count += 1
		    continue
		else:
		    dlg.Destroy()
	    if stock.yahoo() == "Not Available":
		label = "Enter price of %s\n%s" % (
			stock.symbol.upper(), stock.description)
		price_d = dialogs.Price('Price Update', label)
		result = price_d.ShowModal()
		if result == wx.ID_OK:
		    price = price_d.price_numctrl.GetValue()
		    stock.manual_price(price)
		    price_d.Destroy()
		else:
		    price_d.Destroy()
	    stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    count += 1
	    (keepGoing,skip)= dialog.Update(
		    count, "updated %s" % ticker.upper())
	dialog.Destroy()
	self.sync()

    def OnPrice(self, event):
	choices = ['VIX']
	tickers = sorted(self.portfolios.keys())
	for ticker in tickers:
	    choices.append(ticker)
	symbol = wx.GetSingleChoice("Select a portfolio", 
		"portfolio list", 
		choices,
		parent=self)
	if symbol == "VIX":
	    label = "Enter value of VIX\nCBOE VOLATILITY INDEX"
	    price_d = dialogs.Price('VIX Update', label)
	    result = price_d.ShowModal()
	    if result == wx.ID_OK:
		self.cash.vix = price_d.price_numctrl.GetValue()
		self.cash.transform_vix()
		self.statusbar.SetStatusText("Adjusted cash level (%%): %.2f" 
			% self.cash.level, 0)
		self.db['cash']=self.cash
		self.db.sync()
		for ticker in tickers:
		    stock = self.portfolios[ticker]
		    stock.calc(self.cash.level, self.db['delay'], 
			    paramChange=False)
		price_d.Destroy()
	    else:
		price_d.Destroy()
		return
	else:
	    stock = self.portfolios[symbol]
	    if stock.rec:
		dlg = wx.MessageDialog(self, tooltips.update_warning
			% (stock.rec, stock.symbol.upper()),
			"Are you sure?",
			wx.YES_NO|wx.NO_DEFAULT|wx.ICON_QUESTION)
		retCode = dlg.ShowModal()
		if not retCode == wx.ID_YES:
		    dlg.Destroy()
		    return
		else:
		    dlg.Destroy()
	    label = "Enter price of %s\n%s" % (
		    symbol.upper(), stock.description)
	    price_d = dialogs.Price('Price Update', label)
	    result = price_d.ShowModal()
	    if result == wx.ID_OK:
		price = price_d.price_numctrl.GetValue()
		price_d.Destroy()
		stock.manual_price(price)
		stock.calc(self.cash.level, self.db['delay'], paramChange=False)
	    else:
		price_d.Destroy()
		return
	self.sync()

    def OnDetails(self, event):
	try:
	    detailsFrame = details.DetailFrame(
		    self.db, self.config, self.logfile)
	    detailsFrame.Show()
	except:
	    wx.MessageBox("No portfolios set up yet", parent=self) 

    def OnCashLevel(self, event):
	self.cash.core = float(self.sc_cash.GetValue())
	self.cash.transform_vix()
	self.statusbar.SetStatusText("Adjusted cash level (%%): %.2f" 
		% self.cash.level, 0)
	for ticker in sorted(self.portfolios.keys()):
	    stock = self.portfolios[ticker]
	    # need to save portfolio control values because they may change
	    # while cycling through parameter changes
	    buy_pc = stock.buy_portfolio_control
	    sell_pc = stock.sell_portfolio_control
	    stock.calc(self.cash.level, self.db['delay'], 
		    paramChange=True)
	    stock.buy_portfolio_control = buy_pc
	    stock.sell_portfolio_control = sell_pc
	self.db['cash'] = self.cash
	self.sync()

    def OnDelay(self, event):
	self.delay = int(self.sc_delay.GetValue())
	self.db['delay'] = self.delay
	self.db.sync()
	for ticker in sorted(self.portfolios.keys()):
	    stock = self.portfolios[ticker]
	    # need to save portfolio control values because they may change
	    # while cycling through parameter changes
	    buy_pc = stock.buy_portfolio_control
	    sell_pc = stock.sell_portfolio_control
	    stock.calc(self.cash.level, self.db['delay'], paramChange=True)
	    stock.buy_portfolio_control = buy_pc
	    stock.sell_portfolio_control = sell_pc
	self.sync()

    def OnCashFlow(self, event):
	menu_text = self.GetMenuBar().FindItemById(event.GetId()).GetLabel()
	if 'Deposit' in menu_text:
	    cashType = 'Deposit'
	    label = "Enter amount to %s\n($%.2f balance)" % (
		    cashType, self.cash.free)
	elif 'Withdraw' in menu_text:
	    cashType = 'Withdraw'
	    label = "Enter amount to %s\n($%.2f available)" % (
		    cashType, self.cash.free)
	money_d = dialogs.DepositWithdraw(cashType, label)
	if cashType == 'Withdraw':
	    money_d.money_numctrl.SetMax(self.cash.free)
	    money_d.money_numctrl.SetLimited(True)
	result = money_d.ShowModal()
	if result == wx.ID_OK:
	    amount = money_d.money_numctrl.GetValue()
	    money_d.Destroy()
	    if cashType == 'Deposit':
		self.cash.deposit(amount)
	    elif cashType == 'Withdraw':
		self.cash.withdraw(amount)
	    self.db['cash'] = self.cash
	    self.db.sync()
	    self.render.recs(self.recs_html, self.db, self.btn_ids)
	else:
	    money_d.Destroy()

    def OnExit(self, event):
	width, height = self.GetSize()
	sash = self.split.GetSashPosition()
	self.config.SetPath('/Frames')
	self.config.Write('action_width', str(width))
	self.config.Write('action_height', str(height))
	self.config.Write('action_sash', str(sash))
	for win in wx.GetTopLevelWindows():
	    win.Close()

