import wx, wx.html
import os
import logging
import shelve, dbhash
from pkg_resources import resource_filename, get_distribution
import aim, gui

class App(wx.App):
    def OnInit(self):
	self.SetAppName('acclaim')
	self.CreateConfig()
	main_frame = gui.Frame(
		self.config, self.db, self.dbfile, self.logfile, self.help)
	main_frame.Show()
	iconFile = resource_filename(__name__, 'icons/line_chart.ico')
	ib = wx.IconBundle()
	ib.AddIconFromFile(iconFile, wx.BITMAP_TYPE_ANY)
	wx.GetApp().GetTopWindow().SetIcons(ib)
	return True
    
    def CreateConfig(self):
	sp = wx.StandardPaths.Get()
	userData = sp.GetUserDataDir()
	if not os.path.isdir(userData):
	    os.mkdir(userData)
	
	self.config = wx.FileConfig(appName='acclaim',
		vendorName='Glenn Johnson',
		localFilename=os.path.join(userData, 'config'))
	
	logging.basicConfig(level=logging.INFO,
		format='%(asctime)s %(message)s',
		datefmt='%a, %d %b %Y %H:%M:%S',
		filename=os.path.join(userData, 'transaction.log'))
	
	self.dbfile = os.path.join(userData, 'ACCLAIM.db')
	self.logfile = os.path.join(userData, 'transaction.log')
	self.db = shelve.open(self.dbfile)
	
	helpFile = resource_filename(__name__, 'htmlhelp/ACCLAIMdoc.hhp')
	self.help = wx.html.HtmlHelpController(
		style=wx.html.HF_DEFAULT_STYLE ^ wx.html.HF_INDEX)
	self.help.SetTempDir(userData)
	self.help.UseConfig(self.config, rootpath='/Help')
	self.help.AddBook(helpFile)

	try:
	    self.cash = self.db['cash']
	except:
	    # need to set up initial account
	    self.cash = aim.Cash()
	    self.cash.free = 0.00
	    self.cash.core = 50.00
	    self.cash.vix = 20.00
	    self.cash.level = 50.00
	    self.cash.interest_earned = 0.0
	    self.delay = 5
	    self.db['cash'] = self.cash
	    self.db['delay'] = self.delay
	    self.db['portfolios'] = {}
	    self.db.sync()

    def OnExit(self):
	self.config.Flush()
	self.db.close()

def main():
    app = App(0)
    app.MainLoop()

if __name__ == "__main__":
    main()
